/* $OpenBSD: rc4.c,v 1.15 2025/08/17 08:04:25 jsing Exp $ */
/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
 * All rights reserved.
 *
 * This package is an SSL implementation written
 * by Eric Young (eay@cryptsoft.com).
 * The implementation was written so as to conform with Netscapes SSL.
 *
 * This library is free for commercial and non-commercial use as long as
 * the following conditions are aheared to.  The following conditions
 * apply to all code found in this distribution, be it the RC4, RSA,
 * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
 * included with this distribution is covered by the same copyright terms
 * except that the holder is Tim Hudson (tjh@cryptsoft.com).
 *
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.
 * If this package is used in a product, Eric Young should be given attribution
 * as the author of the parts of the library used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    "This product includes cryptographic software written by
 *     Eric Young (eay@cryptsoft.com)"
 *    The word 'cryptographic' can be left out if the rouines from the library
 *    being used are not cryptographic related :-).
 * 4. If you include any Windows specific code (or a derivative thereof) from
 *    the apps directory (application code) you must include an acknowledgement:
 *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
 *
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#include <endian.h>
#include <stdint.h>

#include <openssl/rc4.h>

#include "crypto_arch.h"

#ifdef HAVE_RC4_INTERNAL
void rc4_internal(RC4_KEY *key, size_t len, const uint8_t *in,
    uint8_t *out);

#else
static inline RC4_INT
rc4_step(RC4_INT *d, RC4_INT *x, RC4_INT *y)
{
	RC4_INT tx, ty;

	*x = (*x + 1) & 0xff;
	tx = d[*x];
	*y = (tx + *y) & 0xff;
	d[*x] = ty = d[*y];
	d[*y] = tx;

	return d[(tx + ty) & 0xff];
}

#if BYTE_ORDER == BIG_ENDIAN
static inline uint64_t
rc4_chunk(RC4_INT *d, RC4_INT *x, RC4_INT *y)
{
	uint64_t chunk = 0;
	size_t i;

	for (i = 0; i < 8; i++)
		chunk = chunk << 8 | (uint64_t)rc4_step(d, x, y);

	return chunk;
}

#else
static inline uint64_t
rc4_chunk(RC4_INT *d, RC4_INT *x, RC4_INT *y)
{
	uint64_t chunk = 0;
	size_t i;

	for (i = 0; i < 8; i++)
		chunk |= (uint64_t)rc4_step(d, x, y) << (i * 8);

	return chunk;
}
#endif

static void
rc4_internal(RC4_KEY *key, size_t len, const uint8_t *in, uint8_t *out)
{
	RC4_INT *d, x, y;
	size_t i;

	x = key->x;
	y = key->y;
	d = key->data;

	/* Process uint64_t chunks if 8 byte aligned. */
	if ((((size_t)in | (size_t)out) % 8) == 0) {
		while (len >= 8) {
			*(uint64_t *)out = *(const uint64_t *)in ^ rc4_chunk(d, &x, &y);

			in += 8;
			out += 8;
			len -= 8;
		}
	}

	while (len >= 8) {
		for (i = 0; i < 8; i++)
			out[i] = rc4_step(d, &x, &y) ^ in[i];

		in += 8;
		out += 8;
		len -= 8;
	}
	for (i = 0; i < len; i++)
		out[i] = rc4_step(d, &x, &y) ^ in[i];

	key->x = x;
	key->y = y;
}
#endif

#ifdef HAVE_RC4_SET_KEY_INTERNAL
void rc4_set_key_internal(RC4_KEY *key, int len, const uint8_t *data);

#else
static inline void
rc4_set_key_internal(RC4_KEY *key, int len, const uint8_t *data)
{
	RC4_INT *d, tmp;
	int idx1, idx2;
	int i, j;

	d = key->data;
	key->x = 0;
	key->y = 0;
	idx1 = idx2 = 0;

	for (i = 0; i < 256; i++)
		d[i] = i;
	for (i = 0; i < 256; i += 4) {
		for (j = 0; j < 4; j++) {
			tmp = d[i + j];
			idx2 = (data[idx1] + tmp + idx2) & 0xff;
			d[i + j] = d[idx2];
			d[idx2] = tmp;

			if (++idx1 == len)
				idx1 = 0;
		}
	}
}
#endif

void
RC4(RC4_KEY *key, size_t len, const unsigned char *indata,
    unsigned char *outdata)
{
	rc4_internal(key, len, indata, outdata);
}
LCRYPTO_ALIAS(RC4);

void
RC4_set_key(RC4_KEY *key, int len, const unsigned char *data)
{
	rc4_set_key_internal(key, len, data);
}
LCRYPTO_ALIAS(RC4_set_key);
